@@ -54,10 +54,6 @@ class GitlabConfig
     @config['audit_usernames'] ||= false
   end
 
-  def git_annex_enabled?
-    @config['git_annex_enabled'] ||= false
-  end
-
   def git_trace_log_file
     @config['git_trace_log_file']
   end
